﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Globalization;
using System.Reflection;
using Produmex.Foundation.Data.DbClient;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.ViewObjects.Definitions.Views;

// PmxWorkflowExecutionTypes.HOOK_FLOW

namespace WorkflowScript_BeforeEnterQuantityConsolidatedAdHocMoveHookScript
{
    /// <summary>
    /// This is a hook script, meant to be customized on a "per project" basis
    /// </summary>
    public class WorkflowScript_BeforeEnterQuantityConsolidatedAdHocMoveHookScript : WorkflowInstanceScriptBase
    {
        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        // Input parameters * do not change *
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;
		public ReadOnlyBinder<List<MoveItemInfoExtended>> MoveItemInfoList;

		// Output parameters * do not change *
		public ReadWriteBinder<bool> BackRequested;

        // Sub-flows
        // <none>

        /// <summary>
        /// Initializes a new instance of the <see cref="WorkflowScript_BeforeEnterQuantityConsolidatedAdHocMoveHookScript"/> class.
        /// </summary>
        /// <param name="parent">The parent.</param>
        /// <param name="factory">The factory.</param>
        public WorkflowScript_BeforeEnterQuantityConsolidatedAdHocMoveHookScript( WorkflowInstanceBase parent, WorkflowInstanceFactory factory )
            : base(parent, factory)
        {
        }

        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {
			// Parameters in scope
			Session session = GetScopeParameter("Session") as Session;
			ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;

			string initialErrorKey = null;
			Message msg = null;

			string query = BuildQuery.GetPossibleLocationsWithFreeQuantity(MoveItemInfoList.Get()[0].ItemCode, MoveItemInfoList.Get()[0].SourcePmxWhsCode, sboProviderService.GetDbTool());
			DataSet dsLocationsAndFreeQuantity = sboProviderService.RunView(false, null, null, query);

			DataSet dsShowLocationAndQuantity = BuildMoveItemInfoList(MoveItemInfoList.Get()[0].ItemCode, dsLocationsAndFreeQuantity);

			if (GetNrOfRows(dsShowLocationAndQuantity) > 0)
			{
				session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Picking.ISelectAlternateItemScreen),
					this.DefaultCultureInfo, BuildParamCollection(
						"InitialErrorKey", initialErrorKey,
						"TitleKey", "MSG_TITLE_DETAIL_OF_ITEMS",
						"DSAlternateItem", dsShowLocationAndQuantity));

				msg = WaitForMessage();

				if (msg.Name.EndsWith(".BackRequested"))
				{
					BackRequested.Set(true);
				}

			}

		}

		#endregion

		#region HELPERS

		private int GetNrOfRows(DataSet ds)
		{
			return (ds != null && ds.Tables.Count > 0) ? ds.Tables[0].Rows.Count : 0;
		}

		private DataSet BuildMoveItemInfoList(string itemCode, DataSet moveItemInfoList)
		{
			DataSet dsItemOnStorageLocations = new DataSet();

			DataTable tblItemOnStorageLocations = dsItemOnStorageLocations.Tables.Add("MoveItemInfo");

			tblItemOnStorageLocations.Columns.Add(AlternateItemDataSet.Columns.INTERNAL_KEY.Name, typeof(int));
			tblItemOnStorageLocations.Columns.Add(AlternateItemDataSet.Columns.STORAGE_LOCATION.Name);
			tblItemOnStorageLocations.Columns.Add(AlternateItemDataSet.Columns.QUANTITY.Name, typeof(double));
			tblItemOnStorageLocations.Columns.Add(AlternateItemDataSet.Columns.ITEMCODE.Name);
			tblItemOnStorageLocations.Columns.Add(AlternateItemDataSet.Columns.BESTBEFOREDATE.Name, typeof(DateTime));
			tblItemOnStorageLocations.Columns.Add(AlternateItemDataSet.Columns.BATCHNUMBER.Name);
			tblItemOnStorageLocations.Columns.Add(AlternateItemDataSet.Columns.UOM.Name);
			tblItemOnStorageLocations.Columns.Add(AlternateItemDataSet.Columns.ITEM_DESCRIPTION.Name);
			tblItemOnStorageLocations.Columns.Add(AlternateItemDataSet.Columns.ITEM_BARCODE.Name);
			tblItemOnStorageLocations.Columns.Add(AlternateItemDataSet.Columns.SSCC.Name);
			tblItemOnStorageLocations.Columns.Add(AlternateItemDataSet.Columns.UOM2.Name);
			tblItemOnStorageLocations.Columns.Add(AlternateItemDataSet.Columns.QUANTITY_UOM2.Name, typeof(double));
			tblItemOnStorageLocations.Columns.Add(AlternateItemDataSet.Columns.BATCHNUMBER2.Name);
			tblItemOnStorageLocations.Columns.Add(AlternateItemDataSet.Columns.MASTER_SSCC.Name);

			if (moveItemInfoList != null && moveItemInfoList.Tables[0].Rows.Count > 0)
			{
				for (int i = 0; i < moveItemInfoList.Tables[0].Rows.Count; i++)
				{
					DataRow row = tblItemOnStorageLocations.Rows.Add(i,
						moveItemInfoList.Tables[0].Rows[i].ItemArray[0].ToString(),	// 0 - LocationCode
						moveItemInfoList.Tables[0].Rows[i].ItemArray[1].ToString(), // 1 - Quantity
						itemCode,
						null,
						null,
						null,
						itemCode,
						null,
						null,
						null,
						null,
						null,
						null
					);
				}
			}

			return dsItemOnStorageLocations;
		}

		#endregion

		#region BUILD QUERY

		internal static class BuildQuery
		{
			public static string GetPossibleLocationsWithFreeQuantity(string itemCode, string warehouseCode, DbTool dbTool)
			{
				DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();

				strBuilder.
					Select.
						EscTableColumnName(PmxFreeStockViewDefinition.NAME, PmxFreeStockViewDefinition.Columns.StorLocCode.NAME).Comma.
						Append("SUM(", SqlCommandHelper.EscapeTableColumnName(PmxFreeStockViewDefinition.NAME, PmxFreeStockViewDefinition.Columns.FreeQuantityDetail.NAME), ")").
						As.EscColumnName(AlternateItemDataSet.Columns.QUANTITY.Name).CRLF.
					From.
						EscTableName(PmxFreeStockViewDefinition.NAME).CRLF.
					InnerJoin.
						EscTableName(PmxOseGeneralTable.NAME).
						On.
							EscTableColumnName(PmxFreeStockViewDefinition.NAME, PmxFreeStockViewDefinition.Columns.StorLocCode.NAME).Eq.
							EscTableColumnName(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME).CRLF.
					Where.
						EscTableColumnName(PmxFreeStockViewDefinition.NAME, PmxFreeStockViewDefinition.Columns.ItemCode.NAME).Eq.
						Append(SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture)).
					And.
						EscTableColumnName(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME).Eq.
						Append(SqlCommandHelper.FormatAndEscapeColumnValue(warehouseCode, CultureInfo.InvariantCulture)).
					GroupBy.
						EscTableColumnName(PmxFreeStockViewDefinition.NAME, PmxFreeStockViewDefinition.Columns.StorLocCode.NAME);


				return strBuilder.ToString();
			}
		}

		#endregion
	}
}
